#include "MeshLoader.h"
using namespace std;

map<int, Node>& MeshLoader::Get_Nodes() {
    return nodes;
}
vector<FiniteElement>& MeshLoader::Get_FiniteElements() {
    return finiteelements;
}
vector<BoundaryFiniteElement>& MeshLoader::Get_BFEs() {
    return bfes;
}
vector<FiniteElement> MeshLoader::Get_FE_by_nodes(int id1, int id2, int id3) {
    vector<FiniteElement> ans;
    auto it = finiteelements.begin();
    while (it != finiteelements.end()){
    it = find_if(it, finiteelements.end(), [id1, id2, id3](FiniteElement x) { //!!! Копирование x на каждой итерации внутри find_if 
        return ((find(x.nodes_id.begin(), x.nodes_id.end(), id1) != x.nodes_id.end()) &&
                (find(x.nodes_id.begin(), x.nodes_id.end(), id2) != x.nodes_id.end()) &&
                (find(x.nodes_id.begin(), x.nodes_id.end(), id3) != x.nodes_id.end())
        );
    });
    if (it != finiteelements.end()) {
        ans.push_back(*it);
        it++;
    }
}
    return ans;
}
vector<FiniteElement> MeshLoader::Get_FE_by_edge(int id1, int id2) {
    vector<FiniteElement> ans;
    auto it = finiteelements.begin();
    while (it != finiteelements.end()) {
        it = find_if(it, finiteelements.end(), [id1, id2](FiniteElement x)  { //!!! Копирование x на каждой итерации внутри find_if 
            return ((find(x.nodes_id.begin(), x.nodes_id.end(), id1) != x.nodes_id.end()) &&
                    (find(x.nodes_id.begin(), x.nodes_id.end(), id2) != x.nodes_id.end())
            );
        });
        if (it != finiteelements.end()) {
            ans.push_back(*it);
            it++;
        }
    }
    return ans;
}

vector<Node> MeshLoader::Get_nodes_by_border(int id) {
    vector <Node> ans;
    auto it = bfes.begin();
    while(it != bfes.end()){
        it = find_if(it, bfes.end(), [id](BoundaryFiniteElement x){ //!!! Копирование x на каждой итерации внутри find_if 
            return x.border_id = id;});
        for (const auto& x : it->nodes_id)
            ans.push_back(nodes[x]);
        it++;
    }
    return ans;
}
vector<FiniteElement>MeshLoader::Get_FE_by_material(int id) {
    vector<FiniteElement> ans;
    auto it = finiteelements.begin();
    while (it != finiteelements.end()) {
        it = find_if(it, finiteelements.end(), [id](FiniteElement x) { //!!! Копирование x на каждой итерации внутри find_if 
            return x.mat_id == id; });
        if (it != finiteelements.end()) {
            ans.push_back(*it);
            it++;
        }
    }
    return ans;
}
vector <BoundaryFiniteElement>MeshLoader::Get_BFE_by_border(int id) {
    vector<BoundaryFiniteElement> ans;
    auto it = bfes.begin();
    while (it != bfes.end()) {
        it = find_if(it, bfes.end(), [id](BoundaryFiniteElement x) { //!!! Копирование x на каждой итерации внутри find_if 
            return x.border_id == id; });
        if (it != bfes.end()) {
            ans.push_back(*it);
            it++;
        }
    }
    return ans;
}

//!!! Узлы нужно вставлять сразу во все элементы
//!!! При этом учитывая, что на ребре данного элемента ранее мог быть создан узел при обходе соседнего элемента.
void MeshLoader::Insertion(Node node, FiniteElement& fe) { //!!! node по ссылке на константу
    int p = fe.nodes_id.size() / 2;
    auto it = fe.nodes_id.begin();
    advance(it, p);
    fe.nodes_id.insert(it, node.id);
    nodes.insert({node.id, node});
}
/*Node MeshLoader::Get_node_by_id(vector <Node> x, int id)
{
    auto it = find_if(x.begin(), x.end(), [&](Node x){
        return x.id == id;
    });
    if (it == x.end())
    {
        Node* t = new Node;
        t->id = -100;
        return *t;
    }
    else
        return *it;
}*/
map <int, set<Node>> MeshLoader::Get_neighbors(){
    
    //!!! Лучше std::vector 
    map <int, set <Node>> ans;
    for (const auto& itfe : finiteelements)
    {
        for(const auto& id : itfe.nodes_id)
            for(const auto& id1 : itfe.nodes_id)
                    ans[id].insert(nodes[id1]);
    }
    for (auto& x : ans)
        x.second.erase(nodes[x.first]);
    return ans;
}

void MeshLoader::printNode(const Node &node) {
    std::cout << node;
}

void MeshLoader::printFE(const FiniteElement &fe) {
    std::cout << fe;
}

void MeshLoader::printBFE(const BoundaryFiniteElement &bfe) {
    std::cout << bfe;
}

vector<int> MeshLoader::getBoundaryNodesId() {
    std::vector<int> res;
    for (const auto &it: bfes) {
        std::for_each(it.nodes_id.begin(), it.nodes_id.end(), [&res](int id) {
            res.push_back(id);
        });
    }
    return res;
}


